import {
  ConfigurationDomain,
  ConfigurationIdentifier,
  ConfigurationType,
  ConfigurationValue,
} from '@mobicoop/configuration-module';
import { NotFoundException } from '@mobicoop/ddd-library';
import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { CarpoolConfig } from '@src/config/carpool.config';
import { Config } from '@src/config/config';
import { MatchConfig } from '@src/config/match.config';
import { PaginationConfig } from '@src/config/pagination.config';

@Injectable()
export class ConfigurationsManagerService {
  constructor(private readonly configService: ConfigService) {}

  list = (): Config[] => {
    return [
      {
        ...(this.configService.get<CarpoolConfig>('carpool') as CarpoolConfig),
        domain: ConfigurationDomain.CARPOOL,
      },
      {
        ...(this.configService.get<MatchConfig>('match') as MatchConfig),
        domain: ConfigurationDomain.MATCH,
      },
      {
        ...(this.configService.get<PaginationConfig>(
          'pagination',
        ) as PaginationConfig),
        domain: ConfigurationDomain.PAGINATION,
      },
    ];
  };

  identifierType = (
    configurationIdentifier: ConfigurationIdentifier,
  ): ConfigurationType => {
    const configs: Config[] = this.list();
    const configuration: Config | undefined = configs.find(
      (config: Config) =>
        config.domain === configurationIdentifier.domain &&
        this._hasProperty(configurationIdentifier.key, config),
    );
    if (!configuration)
      throw new NotFoundException('Configuration item not found');
    return this.configurationType(
      this._getValue(configurationIdentifier.key, configuration),
    );
  };

  configurationType = (value: any): ConfigurationType => {
    switch (typeof value) {
      case 'number':
        if (this._isInt(value)) return ConfigurationType.INT;
        return ConfigurationType.FLOAT;
      case 'boolean':
        return ConfigurationType.BOOLEAN;
      default:
        return ConfigurationType.STRING;
    }
  };

  cast = (
    value: string,
    configurationType: ConfigurationType,
  ): ConfigurationValue => {
    switch (configurationType) {
      case ConfigurationType.BOOLEAN:
        return value === 'true';
      case ConfigurationType.INT:
        return parseInt(value);
      case ConfigurationType.FLOAT:
        return parseFloat(value);
      default:
        return value;
    }
  };

  private _hasProperty = <T>(property: string, configuration: T): boolean => {
    let key: keyof typeof configuration;
    for (key in configuration) {
      if (key !== 'domain' && key === property) return true;
    }
    return false;
  };

  private _getValue = <T>(property: string, configuration: T): any => {
    let key: keyof typeof configuration;
    for (key in configuration) {
      if (key !== 'domain' && key === property) return configuration[key];
    }
  };

  private _isInt = (number: number): boolean => number % 1 === 0;
}
