import {
  ConfigurationDomain,
  ConfigurationIdentifier,
  ConfigurationType,
} from '@mobicoop/configuration-module';
import { NotFoundException } from '@mobicoop/ddd-library';
import { ConfigurationsManagerService } from '@modules/configuration/core/application/services/configurations-manager.service';
import { ConfigService } from '@nestjs/config';
import { Test, TestingModule } from '@nestjs/testing';
import { Config } from '@src/config/config';

const mockConfigService = {
  get: jest.fn().mockImplementation((domain: string) => {
    switch (domain) {
      case 'carpool':
        return {
          departureTimeMargin: 900,
          role: 'passenger',
          seatsProposed: 3,
          seatsRequested: 1,
          strictFrequency: false,
        };
      case 'match':
        return {
          algorithm: 'PASSENGER_ORIENTED',
          remoteness: 15000,
          useProportion: true,
          proportion: 0.3,
          useAzimuth: true,
          azimuthMargin: 10,
          maxDetourDistanceRatio: 0.3,
          maxDetourDurationRatio: 0.3,
        };
      case 'pagination':
        return {
          perPage: 10,
        };
    }
  }),
};

describe('Configurations Manager Service', () => {
  let configurationsManagerService: ConfigurationsManagerService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: ConfigService,
          useValue: mockConfigService,
        },
        ConfigurationsManagerService,
      ],
    }).compile();

    configurationsManagerService = module.get<ConfigurationsManagerService>(
      ConfigurationsManagerService,
    );
  });

  it('should be defined', () => {
    expect(configurationsManagerService).toBeDefined();
  });

  it('should return the list of configuration elements', () => {
    const list: Config[] = configurationsManagerService.list();
    expect(list).toHaveLength(3);
  });

  describe('identifierType', () => {
    it('should return the type of a configuration item for a given identifier', () => {
      const configurationIdentifier: ConfigurationIdentifier = {
        domain: ConfigurationDomain.CARPOOL,
        key: 'seatsProposed',
      };
      const configurationType: ConfigurationType =
        configurationsManagerService.identifierType(configurationIdentifier);
      expect(configurationType).toBe(ConfigurationType.INT);
    });
    it('should throw if configuration item is not found', () => {
      const configurationIdentifier: ConfigurationIdentifier = {
        domain: ConfigurationDomain.MATCH,
        key: 'maxDetour',
      };
      expect(() => {
        configurationsManagerService.identifierType(configurationIdentifier);
      }).toThrow(NotFoundException);
    });
  });

  describe('configurationType', () => {
    it('should return the configuration type of an int', () => {
      expect(configurationsManagerService.configurationType(3)).toBe(
        ConfigurationType.INT,
      );
    });
    it('should return the configuration type of a float', () => {
      expect(configurationsManagerService.configurationType(3.5)).toBe(
        ConfigurationType.FLOAT,
      );
    });
    it('should return the configuration type of a boolean', () => {
      expect(configurationsManagerService.configurationType(true)).toBe(
        ConfigurationType.BOOLEAN,
      );
    });
    it('should return the configuration type of a string', () => {
      expect(configurationsManagerService.configurationType('role')).toBe(
        ConfigurationType.STRING,
      );
    });
  });

  describe('cast', () => {
    it('should cast a string to int', () => {
      expect(
        configurationsManagerService.cast('1', ConfigurationType.INT),
      ).toBe(1);
    });
    it('should cast a string to float', () => {
      expect(
        configurationsManagerService.cast('1.5', ConfigurationType.FLOAT),
      ).toBe(1.5);
    });
    it('should cast a string to boolean', () => {
      expect(
        configurationsManagerService.cast('true', ConfigurationType.BOOLEAN),
      ).toBeTruthy();
    });
    it('should not cast a string and return it as is', () => {
      expect(
        configurationsManagerService.cast('role', ConfigurationType.STRING),
      ).toBe('role');
    });
  });
});
